* {
border-color: hsl(0, 0%, 20%) !important;
color: hsl(0, 0%, 60%) !important;
text-shadow: none !important;
}
div[id]{
background-image: none !important
}
*:not(:empty):not(button):not([onclick*="open"]):not(a):not(i):not([class*="stars"]):not([id*="stars"]):not([id="rating"]):not([class="rating"]):not([class*="SPRITE"]):not([id*="SPRITE"]):not([id*="lbImage"]:not([href*="logo"]):not([class*="btn"]):not([class*="button"]):not([class*="icon"]))
{ 
background-image: none !important;
background-color: inherit !important ;
}
html, body {
background: none repeat scroll 0 0 hsl(210, 28%, 10%) !important;
}
div, span, applet, object, iframe, p, blockquote, pre, a, abbr, acronym, address, big, cite, code, del, dfn, em, font, img, ins, kbd, q, s, samp, small, strike, strong, sub, sup, tt, var, b, u, i, center, dl, dt, dd, ol, ul, li, fieldset, form, label, legend, table, caption, tbody, tfoot, thead, tr, th, td {
color: hsl(169, 76%, 36%) !important;
background-color: inherit !important ;
}
span {
color: hsl(207, 7%, 76%) !important;
}
.fbJewel a.jewelButton .jewelButtonHcm {
display: block;
}
[style~="position"], .modal-content {
background-color: hsl(0, 0%, 26%);
box-shadow: 0 0 5px hsl(0, 0%, 80%);
color: hsl(0, 0%, 80%);
}
a {
color: hsl(145, 63%, 49%) !important;
text-decoration: none !important;
text-shadow: none !important;
}
a:hover {
background-image: inherit !important;
color: hsl(67, 100%, 30%) !important;
text-decoration: none !important;
text-shadow: none !important;
}
h1, h2, h3, h4, h5, h6 {
background-image: none !important;
color: hsl(0, 0%, 100%) !important;
text-decoration: none !important;
text-shadow: none !important;
}
img:hover, object:hover {
opacity: 1 !important;
}
a:hover img {
opacity: 1 !important;
}
li {
color: hsl(0, 67%, 40%) !important;
}
body {
color: hsl(0, 0%, 20%) !important;
font-family: "Helvetica Neue",Helvetica,Arial,sans-serif !important;
font-size: 14px !important;
line-height: 1.42857 !important;
}
#navigation li.active a:before {
border-color: hsla(0, 0%, 100%, 0) hsla(0, 0%, 100%, 0) hsl(0, 0%, 0%) !important;
}
#navigation {
background-color: hsl(0, 0%, 13%) !important;
}
#main-content .portlet-boundary_58_.portlet-boundary {
background-color: hsl(0, 0%, 25%) !important;
}
#main-content .portlet-boundary.portlet-borderless .portlet-topper {
background: none repeat scroll 0 0 hsl(0, 0%, 26%) !important;
}
#main-content .portlet-boundary {
background: none repeat scroll 0 0 hsl(0, 0%, 10%) !important;
color: hsl(0, 0%, 100%) !important;
}
#main-content .portlet-boundary.portlet-borderless {
background: none repeat scroll 0 0 hsla(0, 0%, 0%, 0) !important;
padding: 0 !important;
}
footer.bs-footer {
background: none repeat scroll 0 0 hsl(0, 0%, 7%) !important;
border-top: 9px solid hsl(0, 0%, 19%) !important;
}
#myAffix ul.nav li.active a {
background: none repeat scroll 0 0 hsl(0, 0%, 25%) !important;
color: hsl(0, 0%, 100%) !important;
}
#myAffix ul.nav li.active:before {
border-color: hsla(0, 0%, 0%, 0) hsla(0, 0%, 0%, 0) hsla(0, 0%, 0%, 0) hsl(0, 0%, 25%) !important;
}
#myAffix ul.nav li {
border-color: hsl(0, 0%, 0%) !important;
}
#myAffix ul.nav li a {
background: none repeat scroll 0 0 hsl(0, 0%, 15%) !important;
border-color: hsl(0, 0%, 0%) !important;
}
.panel {
background: none repeat scroll 0 0 hsl(0, 0%, 14%) !important;
}
.form-inline .panel-title {
color: hsl(0, 0%, 74%) !important;
}
.portlet-body .aui-field-label, .portlet-body .aui-field-label-inline-label, .portlet-body label {
color: hsl(180, 100%, 86%) !important;
}
#main-content .form-control, #main-content .aui-field-input-text {
border-color: hsl(0, 0%, 0%) !important;
color: hsl(0, 0%, 100%) !important;
}
.input-group .form-control {
background: none repeat scroll 0 0 hsl(0, 0%, 20%) !important;
border-color: hsl(0, 0%, 0%) !important;
margin-bottom: 0 !important;
width: 100% !important;
}
.btn-default, button, input[type="submit"], [role="button"] {
background-color: #ACACA0;
border-color: hsl(0, 0%, 0%) !important;
box-shadow: none !important;
color: hsl(0, 0%, 33%) !important;
text-shadow: none !important;
}
table td, table {
background: none repeat scroll 0 0 hsla(0, 0%, 0%, 0) !important;
border-color: hsl(0, 0%, 0%) !important;
color: hsl(0, 0%, 100%) !important;
}
table th {
background: none repeat scroll 0 0 hsl(0, 0%, 7%) !important;
border-color: hsl(0, 0%, 0%) !important;
}
table tbody tr:nth-child(2n+1) {
background: none repeat scroll 0 0 hsl(0, 0%, 22%) !important;
}
table tbody tr {
background: none repeat scroll 0 0 hsl(240, 7%, 35%) !important;
}
.portlet-body .btn-primary {
background: none repeat scroll 0 0 hsl(0, 0%, 9%) !important;
}
.portlet-msg-error, .lfr-message-error {
background-color: hsl(0, 45%, 18%) !important;
border-color: hsl(1, 44%, 46%) !important;
color: hsl(0, 39%, 91%) !important;
}
.portlet-msg-info, .portlet-msg-help, .lfr-message-help, .portlet-msg-progress, .lfr-message-progress, .lfr-message-info {
background-color: hsl(198, 49%, 15%) !important;
border-color: hsl(200, 49%, 38%) !important;
color: hsl(188, 61%, 84%) !important;
}
.well {
background: none repeat scroll 0 0 hsl(0, 0%, 10%) !important;
border-color: hsl(0, 0%, 0%) !important;
}
table > tbody > tr:nth-child(2n+1) > td, table > tbody > tr:nth-child(2n+1) > th {
background: none repeat scroll 0 0 hsla(0, 0%, 0%, 0) !important;
}
.navbar.navbar-default {
background: none repeat scroll 0 0 hsl(0, 0%, 25%) !important;
border: medium none hsl(0, 0%, 0%) !important;
}
.bs-example {
background: none repeat scroll 0 0 hsl(0, 0%, 0%) !important;
border-color: hsl(0, 0%, 40%) !important;
}
.pagination > li > a, .pagination > li > span {
background: none repeat scroll 0 0 hsl(0, 0%, 17%) !important;
border-color: hsl(0, 0%, 0%) !important;
color: hsl(0, 0%, 73%) !important;
}
.alert-danger {
background: none repeat scroll 0 0 hsl(0, 45%, 18%) !important;
border-color: hsl(1, 44%, 46%) !important;
color: hsl(0, 39%, 91%) !important;
}
.alert-info {
background: none repeat scroll 0 0 hsl(198, 49%, 15%) !important;
border-color: hsl(200, 49%, 38%) !important;
color: hsl(190, 63%, 84%) !important;
}
.alert-success {
background: none repeat scroll 0 0 hsl(120, 33%, 12%) !important;
border-color: hsl(121, 33%, 35%) !important;
color: hsl(92, 44%, 85%) !important;
}
.navbar-collapse.in {
background-color: hsl(222, 100%, 2%) !important;
}
.list-group-item {
background: none repeat scroll 0 0 hsl(0, 0%, 14%) !important;
border-color: hsl(0, 0%, 0%) !important;
}
textarea, input[type="text"] {
background: none repeat scroll 0 0 hsl(0, 0%, 22%) !important;
color: hsl(0, 0%, 100%) !important;
}
header {
background: none repeat scroll 0 0 hsl(0, 0%, 16%) !important;
box-shadow: 0 0 5px 0 hsl(0, 0%, 0%) inset !important;
}
section {
background: none repeat scroll 0 0 hsl(0, 0%, 19%) !important;
box-shadow: 0 0 2px hsl(0, 0%, 0%);
padding: 10px;
}
footer {
background: none repeat scroll 0 0 hsl(0, 0%, 0%) !important;
color: hsl(0, 0%, 100%) !important;
}
.slim {
background-color: hsl(0, 0%, 0%) !important;
position: fixed;
top: 0;
width: 100%;
z-index: 2;
}
main {
background-color: hsl(211, 28%, 15%) !important;
background-image: none !important;
}
#mainContainer {
background-color: hsl(211, 28%, 15%) !important;
display: table;
margin-top: 63px !important;
padding: 20px;
}
img, a img, object {
opacity: 0.1 !important;
}
#main-content .portlet-title, #main-content .list-chart a, .portlet-body .portlet-content a.btn, .navbar-default .navbar-nav > li > a {
color: hsl(0, 0%, 100%) !important;
}
.navbar-default .navbar-nav > .active > a, .navbar-default .navbar-nav > .active > a:hover, .navbar-default .navbar-nav > .active > a:focus, .jumbotron, .pagination > .active > a, .pagination > .active > span, .pagination > .active > a:hover, .pagination > .active > span:hover, .pagination > .active > a:focus, .pagination > .active > span:focus, .btn:active, .btn.active {
background: none repeat scroll 0 0 hsl(0, 0%, 0%) !important;
}
._kso.fsm.direction_ltr._55r0 {
background: none repeat scroll 0 0 hsl(0, 0%, 0%) !important;
}
.gbqfqwb {
background: none repeat scroll 0 0 hsl(0, 0%, 22%) !important;
}
.profile-header {
  background-image: none !important;
}
